import math
from collections.abc import Callable


def _mm_ss(sec: float) -> str:
    minute: float
    sec: float
    minute, sec = divmod(sec, 60)
    return f"{int(minute):02}:{int(sec):02}"


def _hh_mm_ss(sec: float) -> str:
    hour: float
    sec: float
    hour, sec = divmod(sec, 3600)
    return f"{int(hour):02}:{_mm_ss(sec)}"


def _d_hh_mm_ss(sec: float) -> str:
    day: float
    sec: float
    day, sec = divmod(sec, 86400)
    return f"{int(day)}d,{_hh_mm_ss(sec)}"


DEFAULT_TEMPLATES: list[tuple[float, Callable[[float], str]]] = [
    (1e-9, lambda sec: f"{sec * 1e12:#.3g} ps"),
    (1e-6, lambda sec: f"{sec * 1e9:#.3g} ns"),
    (1e-3, lambda sec: f"{sec * 1e6:#.3g} µs"),
    (1.0, lambda sec: f"{sec * 1e3:#.3g} ms"),
    (60, lambda sec: f"{sec:#.3g} s"),
    (3600, _mm_ss),
    (360000, _hh_mm_ss),
    (math.inf, _d_hh_mm_ss),
]


def pretty_duration(sec: float) -> str:
    for threshold, template in DEFAULT_TEMPLATES:
        if sec < threshold:
            return template(sec)
    raise ValueError(sec)



from pathlib import Path

import platformdirs
import pydantic
import pydantic_settings as ps

from ._log_level import LogLevel
from ._paths import paths


def joblib_memory_location() -> Path:
    return platformdirs.user_cache_path(appname="joblib")


class Config(ps.BaseSettings):
    model_config = ps.SettingsConfigDict(env_prefix="LIBLAF_GRAPES_")

    joblib_memory_bytes_limit: int | str | None = pydantic.Field(default="4G")

    joblib_memory_location: Path = pydantic.Field(
        default_factory=joblib_memory_location
    )

    log_file: Path = pydantic.Field(default=paths.log_file)

    log_level: LogLevel = pydantic.Field(default=LogLevel.INFO)


config = Config()



from . import (
    conf,
    enum,
    env,
    error,
    functools,
    git,
    human,
    itertools,
    logging,
    path,
    pretty,
    sentinel,
    serde,
    text,
    timing,
    tqdm,
    typed,
)
from ._version import __version__, __version_tuple__, version, version_tuple
from .conf import Config, LogLevel, Paths, config, paths
from .deps import has_module, optional_imports, try_import
from .enum import CaseInsensitiveEnum
from .env import init_env
from .error import MatchError
from .functools import (
    ConditionalDispatcher,
    Decorator,
    MemorizedFunc,
    cache,
    decorator,
    unbind,
    unbind_getattr,
)
from .human import (
    human_count,
    human_duration,
    human_duration_with_stdev,
    human_throughput,
)
from .itertools import (
    IterableWrapper,
    as_iterable,
    as_sequence,
    first_not_none,
    generator_to_list,
)
from .path import as_path, is_path_like, project_root, resolve_project_path
from .pretty import (
    get_console,
    has_ansi,
    pdoc_attrs,
    pformat,
    pformat_attrs,
    pretty_call,
    pretty_func,
    rich_location,
    wadler_lindig,
)
from .sentinel import MISSING, NOP, nop
from .serde import json, load, load_pydantic, save, save_pydantic, toml, yaml
from .text import strip_comments
from .timing import BaseTimer, ClockName, Timer, Timings, clock, get_timer, timer
from .tqdm import Progress, RateColumn, len_safe, parallel, track
from .typed import ClassInfo, PathLike, clone_param_spec, clone_signature

__all__ = [
    "MISSING",
    "NOP",
    "BaseTimer",
    "CaseInsensitiveEnum",
    "ClassInfo",
    "ClockName",
    "ConditionalDispatcher",
    "Config",
    "Decorator",
    "IterableWrapper",
    "LogLevel",
    "MatchError",
    "MemorizedFunc",
    "PathLike",
    "Paths",
    "Progress",
    "RateColumn",
    "Timer",
    "Timings",
    "__version__",
    "__version_tuple__",
    "as_iterable",
    "as_path",
    "as_sequence",
    "cache",
    "clock",
    "clone_param_spec",
    "clone_signature",
    "conf",
    "config",
    "decorator",
    "enum",
    "env",
    "error",
    "first_not_none",
    "functools",
    "generator_to_list",
    "get_console",
    "get_timer",
    "git",
    "has_ansi",
    "has_module",
    "human",
    "human_count",
    "human_duration",
    "human_duration_with_stdev",
    "human_throughput",
    "init_env",
    "is_path_like",
    "itertools",
    "json",
    "len_safe",
    "load",
    "load_pydantic",
    "logging",
    "nop",
    "optional_imports",
    "parallel",
    "path",
    "paths",
    "pdoc_attrs",
    "pformat",
    "pformat_attrs",
    "pretty",
    "pretty_call",
    "pretty_func",
    "project_root",
    "resolve_project_path",
    "rich_location",
    "save",
    "save_pydantic",
    "sentinel",
    "serde",
    "strip_comments",
    "text",
    "timer",
    "timing",
    "toml",
    "tqdm",
    "track",
    "try_import",
    "typed",
    "unbind",
    "unbind_getattr",
    "version",
    "version_tuple",
    "wadler_lindig",
    "yaml",
]



import enum
from typing import Any, override


class CaseInsensitiveEnum(enum.StrEnum):
    @override
    @staticmethod
    def _generate_next_value_(
        name: str, start: int, count: int, last_values: list[Any]
    ) -> Any:
        return name.upper()

    @override
    @classmethod
    def _missing_(cls, value: object) -> Any:
        if isinstance(value, str):
            value = value.upper()
        for member in cls:
            if member.value == value:
                return member
        return super()._missing_(value)
